/*********************************************
file name: led_intt.c
BY RealSYS KCO	2000.12.31

function: LED shift display using timer1 overflow interrupt
reference file : CodeVisionAVR LED Example file

AVR Chip: AT90S2313 (8MHz)
cable connection:
	CN17(1~8):PB <==> CN28(1~8):LED
*********************************************/
#include <90s2313.h>

// quartz crystal frquency [Hz]
#define xtal 8000000

// moving LED frequency [Hz]
#define fmove 10

// the LED on PORTB output 0 will be on
unsigned char led_status=0xfe;

// TIMER1 overflow interrupt service routine
interrupt [TIM1_OVF] void timer1_overflow(void)
{
	// preset again TIMER1
	TCNT1=0x10000-(xtal/1024/fmove);
	// move the LED
	led_status<<=1;
	led_status|=1;
	if (led_status==0xff) led_status=0xfe;
	
	// turn on the LED
	PORTB=led_status;
}

void main(void)
{
	// set the I/O ports
	// all PORTB pins are outputs
	DDRB=0xff;
	// turn on the first LED
	PORTB=led_status;
	
	// init TIMER1
	// TIMER1 is disconnected from pin OC1
	// no PWM
	TCCR1A=0;

	// TIMER1 clock is xtal/1024
	TCCR1B=5;

	// preset TIMER1
	TCNT1=0x10000-(xtal/1024/fmove);

	// clear TIMER1 interrupts flags
	TIFR=0;

	// enable TIMER1 overflow interrupt
	TIMSK=0x80;

	// all other interrupt sources are disabled
	GIMSK=0;
	
	// global enable interrupts
	#asm
	    sei
	#endasm
	
	// the rest is done by TIMER1 overflow interrupts
	while (1);
}
