/*********************************************
file name: led_intt.c
BY RealSYS KCO	2000.12.31

function: LED shift display using timer1 overflow interrupt
reference file : CodeVisionAVR LED Example file

AVR Chip: AT90S2313 (8MHz)
cable connection:
	CN17(1~8):PB <==> CN28(1~8):LED
*********************************************/
#include <90s2313.h>
// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the AT90S2313

#ifndef _90S2313_INCLUDED_
#define _90S2313_INCLUDED_

#pragma used+
sfrb ACSR=8;
sfrb UBRR=9;
sfrb UCR=0xa;
sfrb USR=0xb;
sfrb UDR=0xc;
sfrb PIND=0x10;
sfrb DDRD=0x11;
sfrb PORTD=0x12;
sfrb PINB=0x16;
sfrb DDRB=0x17;
sfrb PORTB=0x18;
sfrb EECR=0x1c;
sfrb EEDR=0x1d;
sfrb EEAR=0x1e;
sfrb WDTCR=0x21;
sfrb ICR1L=0x24;
sfrb ICR1H=0x25;
sfrw ICR1=0x24;   // 16 bit access
sfrb OCR1L=0x2a;
sfrb OCR1H=0x2b;
sfrw OCR1=0x2a;   // 16 bit access
sfrb TCNT1L=0x2c;
sfrb TCNT1H=0x2d;
sfrw TCNT1=0x2c;  // 16 bit access
sfrb TCCR1B=0x2e;
sfrb TCCR1A=0x2f;
sfrb TCNT0=0x32;
sfrb TCCR0=0x33;
sfrb MCUSR=0x34;
sfrb MCUCR=0x35;
sfrb TIFR=0x38;
sfrb TIMSK=0x39;
sfrb GIFR=0x3a;
sfrb GIMSK=0x3b;
sfrb SPL=0x3d;
sfrb SREG=0x3f;
#pragma used-

#define OCR1AL OCR1L
#define OCR1AH OCR1H

// Interrupt vectors definitions

#define EXT_INT0 2
#define EXT_INT1 3
#define TIM1_CAPT 4
#define TIM1_COMP 5
#define TIM1_OVF 6
#define TIM0_OVF 7
#define UART_RXC 8
#define UART_DRE 9
#define UART_TXC 10
#define ANA_COMP 11

#endif


// quartz crystal frquency [Hz]
#define xtal 8000000

// moving LED frequency [Hz]
#define fmove 10

// the LED on PORTB output 0 will be on
unsigned char led_status=0xfe;

// TIMER1 overflow interrupt service routine
interrupt [TIM1_OVF] void timer1_overflow(void)
{
	// preset again TIMER1
	TCNT1=0x10000-(xtal/1024/fmove);
	// move the LED
	led_status<<=1;
	led_status|=1;
	if (led_status==0xff) led_status=0xfe;
	
	// turn on the LED
	PORTB=led_status;
}

void main(void)
{
	// set the I/O ports
	// all PORTB pins are outputs
	DDRB=0xff;
	// turn on the first LED
	PORTB=led_status;
	
	// init TIMER1
	// TIMER1 is disconnected from pin OC1
	// no PWM
	TCCR1A=0;

	// TIMER1 clock is xtal/1024
	TCCR1B=5;

	// preset TIMER1
	TCNT1=0x10000-(xtal/1024/fmove);

	// clear TIMER1 interrupts flags
	TIFR=0;

	// enable TIMER1 overflow interrupt
	TIMSK=0x80;

	// all other interrupt sources are disabled
	GIMSK=0;
	
	// global enable interrupts
	#asm
	    sei
#endasm
	
	// the rest is done by TIMER1 overflow interrupts
	while (1);
}
