/*********************************************
file name: step_mot.c
BY RealSYS KCO	2001.1.1

function: STEP Motor Control using timer compare match
	PC Comm. Program : '0'=STOP ~ '9'=High speed
	
AVR Chip: AT90S2313 (8MHz)

cable connection:
	CN17(1~4):PB <==> CN23(1~8):STEP_mot

	RS232(20P-CPU) <==> PC COM Port
	
	CN19 <==> Step Motor
*********************************************/
#include <90s2313.h>         
#include <delay.h>
#include <stdio.h>

#define	BYTE	unsigned char
#define	WORD	unsigned int

/* UART Status Register */
#define    RXC		7
#define    TXC		6
#define    UDRE		5
#define    FE		4
#define    OVR		3

#define	t1msec		8		/* 1000 / 128 usec = 7.8125 */
#define	t2msec		16		/* 2000 / 128 usec = 15.625 */
#define	t5msec		39		/* 5000 / 128 usec = 39.0625 */
#define	t10msec		78		/* 10000 / 128 usec = 78.125 */
#define	t20msec		156		/* 20000 / 128 usec = 156.25 */
#define	t50msec		391		/* 50000 / 128 usec = 390.625 */
#define	t100msec	781		/* 100000 / 128 usec = 781.25 */
#define	t200msec	1563	/* 200000 / 128 usec = 1562.5 */
#define	t500msec	3906	/* 500000 / 128 usec = 3906.25 */
#define	t1sec		7813	/* 1000000 / 128 usec = 7812.5 */
#define	t2sec		15625	/* 2000000 / 128 usec = 15625 */

#define	HI_BYTE(w)	(w >> 8)
#define	LO_BYTE(w)	(w & 0xff)

BYTE	rxd;   
int 	m_cnt=0;
int		t_cnt0=0,t_cnt1=0;

flash BYTE fbuf[]={"\n\rStep Motor Increment time is =>"};
flash BYTE message[]={"\n\rPress KEY : '0'(STOP) ~ '9'(High)"};

/************ Step motor ***************/
/*BYTE	sm_data[] = { 0x0c,0x09,0x03,0x06 };*/
BYTE	sm_data[] = { 0x01,0x02,0x04,0x08 };

void	step_mot(WORD val);
void  init_port(void){
	PORTB = 0x00;
	PORTD = 0xff;
	DDRB = 0xff;		/* All output dir , 0=IN,1=OUT*/
	DDRD = 0xfe;		/* PD6(O) ~ PD2(O),PD1/TXD(O),PD0/RXD(I) = 0xFE */
}
void	init_timer(void){
	TCCR0 = 4;			/* CK(8MHz)/256 = 31.250kHz : 32usec*/
	TIMSK |= 2;			/* TOIE0 = 1 */
	TCNT0 = -32;		/* 32usec * 32 = 1.024 msec period */

	TCCR1B = 0x0d;		/* CTC1=1(Match clear), CK(8MHz)/1024 = 7.8125kHz : 128usec*/
	TCNT1H = 0;	TCNT1L = 0;
	step_mot(t100msec);
}
void	init_uart(void){
	UBRR = 51;      	/* 9600 bps for 8MHz */
	UCR = 0x98;     	/* RXCIE, RXEN, TXEN*/
}
void	send_char(char d){
	while(!(USR & (1<<UDRE)));
	UDR = d;
}

BYTE	receive_char(void){
	while(!(USR & (1 <<RXC)));
	return UDR;
}

void    send_string(char *p){
char    rd;
    while(rd = *p++) send_char(rd);
}
void    send_fstring(char flash *p){
char    rd;
    while(rd = *p++) send_char(rd);
}

void	step_mot(WORD val){
	if(val == 0){
		TIMSK &= ~(0x40);	/* OCIE1A = 0 ; Timer/Counter1 OCM Interrupt Disable */
		PORTB &= 0xf0;		/* Step_Mot_PIN All off(0) */
	}
	else{
//		TCNT1H = 0;	TCNT1L = 0;       
		TCNT1 = 0;
		
//		OCR1AH = HI_BYTE(val);		OCR1AL = LO_BYTE(val);
		OCR1 = val;
		TIMSK |= 0x40;		/* OCIE1A = 1 ; Timer/Counter1 OCM Interrupt Enable */
	}	
}

interrupt [TIM0_OVF] void TIMER0_OVF0_interrupt(void) 
{  
	TCNT0 = -32;		/* 32usec * 32 = 1.024 msec period */
	t_cnt0++;
}
interrupt [TIM1_COMP] void TIMER1_COMP1_interrupt(void) 
{  
	t_cnt1++;
	PORTB = sm_data[t_cnt1 & 0x03];
}

interrupt [UART_RXC] void UART_RX_interrupt(void) 
{  
	rxd=UDR;
	putsf(fbuf);
	switch(rxd){
		case '0': step_mot(0);
			send_fstring("STOP");
	  		break;
	  	case '1': step_mot(t500msec);
			send_fstring("500msec");
	  		break;
	  	case '2': step_mot(t200msec);
			send_fstring("200msec");
	  		break;
	  	case '3': step_mot(t100msec);
			send_fstring("100msec");
	  		break;
	  	case '4': step_mot(t50msec);
			send_fstring("50msec");
	  		break;
	  	case '5': step_mot(t20msec);
			send_fstring("20msec");
	  		break;
	  	case '6': step_mot(t10msec);
			send_fstring("10msec");
	  		break;
	  	case '7': step_mot(t5msec);
			send_fstring("5msec");
	  		break;
	  	case '8': step_mot(t2msec);
			send_fstring("2msec");
	  		break;
	  	case '9': step_mot(t1msec);
			send_fstring("1msec");
	  		break;
		default : step_mot(0);
			send_fstring("STOP");
	  		break;
	}
}
void main(void){
	init_port();
	init_uart();
	init_timer();
	putsf(message);

	#asm	("sei")		// global enable interrupts
	while(1){
		m_cnt++;
		delay_ms(1000);
	}
}
