/*********************************************
file name: lcd.c
BY RealSYS KCO	2001.1.3

function: A/D ch0 & ch1 Input => LCD display
		A/D : interrupt based scan ch0 ~ ch1

	cable connection:
	CN33(1):VR <==> CN8(1):ADC0
	CN12(1~4):PA <==> CN31(1~4):LCD-con
	CN12(5~8):PA <==> CN32(5~8):LCD-dat	

	Chip type           : ATmega128
	Clock frequency     : 16.000000 MHz
*********************************************/
#include <mega128.h>
#include <delay.h>		// for delay_ms() function
#include <stdio.h>		// for sprintf() function

// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x1B	; Port A
#endasm
#include <lcd.h>		// for LCD function

#define FIRST_ADC_INPUT 0
#define LAST_ADC_INPUT 1
unsigned int adc_data[LAST_ADC_INPUT-FIRST_ADC_INPUT+1];

// ADC interrupt service routine
// with auto input scanning
#pragma savereg-
interrupt [ADC_INT] void adc_isr(void)
{

#asm
    push r26
    push r27
    push r30
    push r31
    in   r30,sreg
    push r30
#endasm

register static unsigned char input_index=0;
// Read the ADC conversion result
adc_data[input_index]=ADCW;
// Select next ADC input
if (++input_index>(LAST_ADC_INPUT-FIRST_ADC_INPUT))
   input_index=0;
#ifdef BANDGAP
ADMUX=input_index+FIRST_ADC_INPUT|0x40;
#else
ADMUX=input_index+FIRST_ADC_INPUT;
#endif
// Start the next ADC conversion
ADCSRA.6=1;
#asm
    pop  r30
    out  sreg,r30
    pop  r31
    pop  r30
    pop  r27
    pop  r26
#endasm
}
#pragma savereg+

// Declare your global variables here

void main(void)
{
// Declare your local variables here
char	sbuf[20];

// Input/Output Ports initialization
// Port A
DDRA=0x00;
PORTA=0x00;

// Port B
DDRB=0x00;
PORTB=0x00;

// Port D
DDRD=0x00;
PORTD=0x00;

// Port E
DDRE=0x00;
PORTE=0x00;

// ADC initialization
// ADC Clock frequency: 187.500 kHz
ADMUX=FIRST_ADC_INPUT;
ADCSRA=0xCD;

// LCD module initialization
lcd_init(16);

// Global enable interrupts
#asm("sei")

while (1)
      {
      // Place your code here  
      lcd_gotoxy(0,0);
      sprintf(sbuf,"ADC0=%u(%03X)",adc_data[0],adc_data[0]);
      lcd_puts(sbuf);

      lcd_gotoxy(0,1);
      sprintf(sbuf,"ADC1=%u(%03X)",adc_data[1],adc_data[1]);
      lcd_puts(sbuf);             
      
      delay_ms(100);

      };
}
