/*********************************************
file name: uart.c
BY RealSYS KCO	2001.1.4

function: 
	- A/D ch0 Input(interrupt based) => LCD display
	- 7SEGMENT display : A/D ch0 value    
	- DC motor speed adjust using PWM2 duty change
	- STEP motor speed adjust using Timer/counter1 Compare Match Interrupt
	- UART : RX & TX interrupt use

	cable connection:
	CN33(1):VR <==> CN8(1):ADC0
	CN12(1~4):PA <==> CN31(1~4):LCD-con
	CN12(5~8):PA <==> CN32(5~8):LCD-dat	 
	
	CN11(1~8):PC <==> CN29(1~8):SEG_data
	CN7(1~4):PD <==> CN27(1~4):SEG_scan
	
	CN4(8):PWM2 <==> CN26(1): DC_mot   
	
	CN5(5~8):PE <==> CN23(1~4): STEP_mot
	
	RCM128_RS232 <==> PC Com port

	Chip type           : ATmega128
	Clock frequency     : 16.000000 MHz
*********************************************/
#include <mega128.h>
#include <delay.h>		// for delay_ms() function
#include <stdio.h>		// for sprintf() function
// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x1B	; Port A
#endasm
#include <lcd.h>		// for LCD function
#define FIRST_ADC_INPUT 0
#define LAST_ADC_INPUT 0

#define	BYTE	unsigned char
#define	WORD	unsigned int

#define	SCAN_OFF (PORTD |= 0x0f)
#define	SCAN0	(PORTD.0 = 0)
#define	SCAN1	(PORTD.1 = 0)
#define	SCAN2	(PORTD.2 = 0)
#define	SCAN3	(PORTD.3 = 0)

#define	SEG_DATA(val)	(PORTC = val)

flash BYTE 	seg_hex[] = /* HEX font */
	{0xc0,0xf9,0xa4,0xb0,0x99,0x92,0x82,0xf8,0x80,0x90,0x88,0x83,0xc6,0xa1,0x86,0x8e};
BYTE	seg_data[4]= { 0xbf,0xbf,0xbf,0xbf };
BYTE	scan_cnt=0;	

int		m_cnt=0,t_cnt1=0;
char	rxd;

/*BYTE	sm_data[] = { 0xc0,0x90,0x30,0x60 };*/
BYTE	sm_data[] = { 0x10,0x20,0x40,0x80 };
                                            
#define	TX_BUF_SIZE	50
#define	TX_START_ID	'!'
#define	TX_START	{ 	(UCSR1A |= 0x40);	\
						(UDR0 = TX_START_ID);	}
#define	TX_STOP		(UCSR1A &= 0xCF)
char	tx_buf[TX_BUF_SIZE];
char	tx_pos=0, tx_end=0;

unsigned int adc_data[LAST_ADC_INPUT-FIRST_ADC_INPUT+1];
// ADC interrupt service routine
// with auto input scanning
#pragma savereg-
interrupt [ADC_INT] void adc_isr(void){
#asm
    push r26
    push r27
    push r30
    push r31
    in   r30,sreg
    push r30
#endasm
register static unsigned char input_index=0;
// Read the ADC conversion result
adc_data[input_index]=ADCW;
// Select next ADC input
if (++input_index>(LAST_ADC_INPUT-FIRST_ADC_INPUT))
   input_index=0;
#ifdef BANDGAP
ADMUX=input_index+FIRST_ADC_INPUT|0x40;
#else
ADMUX=input_index+FIRST_ADC_INPUT;
#endif
// Start the next ADC conversion
ADCSRA.6=1;
#asm
    pop  r30
    out  sreg,r30
    pop  r31
    pop  r30
    pop  r27
    pop  r26
#endasm
}
#pragma savereg+

interrupt [TIM1_COMPA] void TIM1_COMPA_isr(void) 
{ 
int	data;
	data = 1023-adc_data[0] + 10; 
	OCR1A = data;		// Interrupt gen. interval
	
	t_cnt1++;  
	if(adc_data[0]==0) PORTE &= 0x0f;
	else	PORTE = sm_data[t_cnt1 & 0x03];
}
void	init_timer(void){
    TCNT0 = -16;	/* 64usec * 16 = 1.024 msec period */
	TCCR0 = 7;		/* CK(16MHz)/1024 = 15.625kHz : 64usec */
	TIMSK = 1;		/* TOIE0 = 1 */

	OCR1A = 160;		// 10 msec
	TCNT1 = 0;
	TCCR1B = 0x0d;	/* CTC1=1(Match clear), /* CK(16MHz)/1024 = 15.625kHz : 64usec */
	TIMSK |= 0x10;	/* OCIE1A = 1 ; Timer/Counter1 Output Compare Match Interrupt Enable */
	
	TCCR2 = 0x61;	/* Non-inv 8bit PWM, CK */
	OCR2 = 0;
}
void	init_uart(void){
// USART0 initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART0 Receiver: On
// USART0 Transmitter: On
// USART0 Mode: Asynchronous
// USART0 Baud rate: 9600
    UCSR0A=0x00;
    UCSR0B=0xD8;
    UCSR0C=0x06;
    UBRR0H=0x00;
    UBRR0L=0x67;
}
void	write_txbuf_string(char flash *p){
char	rd;
	while(rd = *p++){
		tx_buf[tx_end++] = rd;
		if(tx_end >= TX_BUF_SIZE) tx_end = 0;
	}
}
void	write_txbuf_char(char d){
	tx_buf[tx_end++] = d;
	if(tx_end >= TX_BUF_SIZE) tx_end = 0;
}
interrupt [USART0_TXC] void UART_TXC_isr(void) 
{      
	if(tx_pos != tx_end){
		UDR0 = tx_buf[tx_pos++];
		if(tx_pos >= TX_BUF_SIZE) tx_pos=0;
	}
	else TX_STOP;
}
interrupt [USART0_RXC] void UART_RXC_isr(void) 
{
	rxd=UDR0;				// get rx data
	lcd_gotoxy(11,1);
	lcd_putsf("RXD=");		// disp to LCD
	lcd_putchar(rxd);
	
	write_txbuf_string("Received char is =");
	write_txbuf_char(rxd);
	write_txbuf_string("\n\r");
	TX_START;  
}
interrupt [TIM0_OVF] void TIM0_OVF_isr(void){
	TCNT0 = -16;	/* 64usec * 16 = 1.024 msec period */

	/* 7segment display */
	SCAN_OFF;
	SEG_DATA(seg_data[scan_cnt]);	/* Seg Data out */
	switch(scan_cnt){				// Seg Scan
		case 0:	SCAN0;	break;
		case 1:	SCAN1;	break;
		case 2:	SCAN2;	break;
		case 3:	SCAN3;	break;
		default: break;
	}                                          
	scan_cnt++;					
	if(scan_cnt > 3) scan_cnt = 0;
}
void	disp_seg4(WORD d){
	seg_data[0] = seg_hex[(d>>12) & 0x0f];
	seg_data[1] = seg_hex[(d>>8) & 0x0f];
	seg_data[2] = seg_hex[(d>>4) & 0x0f];
	seg_data[3] = seg_hex[d & 0x0f];
}

void main(void){
// Declare your local variables here
char	sbuf[20];

// Input/Output Ports initialization
// Port B
DDRB=0xff;
PORTB=0x00; 

// Port C
DDRC=0xff;
PORTC=0xff;

// Port D
DDRD=0xff;
PORTD=0xff;

// Port E
DDRE=0xfe;
PORTE=0x0f;

// ADC initialization
// ADC Clock frequency: 187.500 kHz
ADMUX=FIRST_ADC_INPUT;
ADCSRA=0xCD;

// LCD module initialization
lcd_init(16);

init_uart();

// Timer init.
init_timer();

// Global enable interrupts
#asm("sei")

while (1)
	{
	m_cnt++;
	lcd_gotoxy(0,0);
	sprintf(sbuf,"mcnt=%04Xh:%05u",m_cnt,m_cnt);
	lcd_puts(sbuf);

	lcd_gotoxy(0,1);
	sprintf(sbuf,"OCR1A=%4u",OCR1A);
	lcd_puts(sbuf);

	disp_seg4(adc_data[0]);			// 7segment display
	
	OCR2 = (BYTE)(adc_data[0]>>2);	// DC motor speed control

	delay_ms(100);	   				// delay 0.1 sec
	};
}
