/*********************************************
file name: adc_lcd.c
BY RealSYS KCO	2001.1.2
function: A/D Converter test (A/D Input ==> D/A output)
AVR Chip: ATmega8535 (4MHz)
cable connection:
	CN33(1):VR <==> CN3(1):ADC0
	CN2(1~8):PB <==> CN20(1~8):DAC_data
	CN10(5~8):PD <==> CN24(1~4):DAC_cont
	CN9(1~4):PC <==> CN31(1~4):LCD-con
	CN9(5~8):PC <==> CN32(5~8):LCD-dat
	A/D Conv. time check : CN10(4), Active Low
*********************************************/
#include <mega8535.h> 
#include <stdio.h>     
#include <delay.h>

// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x15	; Port C
#endasm
#include <lcd.h>

#define	BYTE	unsigned char
#define	WORD	unsigned int

#define	ADC_START	(ADCSRA.6=1)
#define	ADC_STOP	(ADCSRA.6=0)
#define	ADC_EOC		(ADCSRA.4)

#define	HI_AD_CHECK	(PORTD.3=1)
#define	LO_AD_CHECK	(PORTD.3=0)

#define	DAC_A0	PORTD.4
#define	DAC_A1	PORTD.5
#define	DAC_WR	PORTD.6

// Declare your global variables here
char	sbuf[20];

int 	m_cnt=0;
void  init_port(void){
	PORTD = 0xff;
	DDRB = 0xff;		/* All output dir , 0=IN,1=OUT*/
	DDRD = 0xfe;		/* PD7(O) ~ PD2(O),PD1/TXD(O),PD0/RXD(I) = 0xFE */
}
void	init_adc(void){
	ADMUX = 0x00;		/* ADMUX = 000b */
	ADCSRA = 0x85;		/* ADEN=1,CK/32=125kHz (4MHz)*/
}
void	out_dac(BYTE ch, BYTE data){
	PORTB = data;
	DAC_A0 = ch & 1;
	DAC_A1 = ch & 2;
	DAC_WR = 0;
	DAC_WR = 1;
}
WORD	ad_data0;
BYTE	ad_byte;
void main(void){
	init_port();

	// LCD module initialization
	lcd_init(16);

	init_adc();
	
	lcd_gotoxy(0,0);
	lcd_putsf("A/D Test RealSYS");
	
	while(1){
		ADC_START;				/* A/D conversion start */
		
		LO_AD_CHECK;			/* for A/D conv. time check */
		while(!ADC_EOC);		/* wait until End of Conversion */
		HI_AD_CHECK;			/* for A/D conv. time check */
		
		ad_data0 = ADCW;		/* Read A/D data */

		ad_byte = (BYTE)(ad_data0 >> 2);

		out_dac(0,ad_byte);		/* out to D/A ch0 */
                                    	
		m_cnt++;
		out_dac(1,(BYTE)m_cnt);
		
		sprintf(sbuf,"CH0=%d(%04X)",ad_data0,ad_data0);
		lcd_gotoxy(0,1);
		lcd_puts(sbuf);
		
		delay_ms(200);
	}
}
