/*********************************************
file name: seg1.c
BY RealSYS KCO	2001.1.2

function: 7 segment display, 4digit hex count up

AVR Chip: ATmega8535 (4MHz)
cable connection:
	CN2(1~8):PB <==> CN29(1~8):SEG_Data
	CN10(5~8):PD <==> CN27(1~4):SEG_Scan
*********************************************/
#include <mega8535.h>         
#include <delay.h>

#define	SEG_DATA(val)	(PORTB = val)
#define	SEG_SCAN(val)	(PORTD = val)

#define	BYTE	unsigned char
#define	WORD	unsigned int
int m_cnt=0;
int	t_cnt=0;

BYTE 	seg_hex[] = /* HEX font */
	{0xc0,0xf9,0xa4,0xb0,0x99,0x92,0x82,0xf8,0x80,0x90,0x88,0x83,0xc6,0xa1,0x86,0x8e};
BYTE 	seg_blank = 0xff;		/* ' ' */
BYTE 	seg_bar = 0xbf;			/* '-' */
BYTE 	seg_all_on = 0x00;		/* '8.' */
BYTE 	seg_t = 0x87;			/* 't' */
BYTE 	seg_scan[4]= { 0xe7,0xdf,0xbf,0x7f };
BYTE	seg_data[4]= { 0xbf,0xbf,0xbf,0xbf };
BYTE	scan_cnt=0;	

void  init_port(void){          
	PORTD = 0xff;
	DDRB = 0xff;	/* All output dir , 0=IN,1=OUT*/
	DDRD = 0xfe;	/* PD7(O)~PD2(O),PD1/TXD(O),PD0/RXD(I) = 0xFE */
}
void	init_timer(void){
	TCCR0 = 4;		/* CK(4MHz)/256 = 15.625kHz : 64usec */
	TIMSK = 1;		/* TOIE0 = 1 */
	TCNT0 = -16;	/* 64usec * 16 = 1.024 msec period */
}
interrupt [TIM0_OVF] void TIMER0_OVF0_interrupt(void){ 
	TCNT0 = -16;	/* 64usec * 16 = 1.024 msec period */
	t_cnt++;

	/* 7segment */
	SEG_SCAN(0xff);					/* SEG OFF */
	SEG_DATA(seg_data[scan_cnt]);	/* Seg Data out */
	SEG_SCAN(seg_scan[scan_cnt]); 	/* Seg Scan out */
	
	scan_cnt++;					
	if(scan_cnt > 3) scan_cnt = 0;
}
void	disp_seg4(WORD d){
	seg_data[0] = seg_hex[(d>>12) & 0x0f];
	seg_data[1] = seg_hex[(d>>8) & 0x0f];
	seg_data[2] = seg_hex[(d>>4) & 0x0f];
	seg_data[3] = seg_hex[d & 0x0f];
}

void main(void){    
	init_port();
	init_timer();

// global enable interrupts
#asm
    sei
#endasm
	while(1){
		m_cnt++;
		disp_seg4(m_cnt);    
		delay_ms(500); 		// 0.5 sec delay
	}
}
