/*********************************************
file name: step_mot.c
BY RealSYS KCO	2001.1.2

function: STEP Motor Control using timer compare match
	PC Comm. Program : '0'=STOP ~ '9'=High speed
	
AVR Chip: ATmega8535 (4MHz)

cable connection:
	CN2(1~4):PB <==> CN23(1~8):STEP_mot

	RS232(40P-CPU) <==> PC COM Port
	
	CN19 <==> Step Motor
*********************************************/
#include <mega8535.h>         
#include <delay.h>
#include <stdio.h>

#define	BYTE	unsigned char
#define	WORD	unsigned int

/* UART Status Register */
#define	RXC		7
#define	TXC		6
#define	UDRE	5
#define	FE		4
#define	OVR		3

#define	t1msec		4		/* 1000 / 256 usec = 3.90625 */
#define	t2msec		8		/* 2000 / 256 usec = 7.8125 */
#define	t5msec		20		/* 5000 / 256 usec = 19.53125 */
#define	t10msec		39		/* 10000 / 256 usec = 39.0625 */
#define	t20msec		78		/* 20000 / 256 usec = 78.125 */
#define	t50msec		195		/* 50000 / 256 usec = 195.3125 */
#define	t100msec	391		/* 100000 / 256 usec = 390.625 */
#define	t200msec	781		/* 200000 / 256 usec = 781.25 */
#define	t500msec	1953	/* 500000 / 256 usec = 1953.125 */
#define	t1sec		3906	/* 1000000 / 256 usec = 3906.25 */
#define	t2sec		7813	/* 2000000 / 256 usec = 7812.5 */

BYTE	rxd;   
int 	m_cnt=0;
int		t_cnt1=0;

flash BYTE message[]={"\n\rPress KEY : '0'(STOP) ~ '9'(High)"};

/************ Step motor ***************/
/*BYTE	sm_data[] = { 0x0c,0x09,0x03,0x06 };*/
BYTE	sm_data[] = { 0x01,0x02,0x04,0x08 };

void	step_mot(WORD val);
void  init_port(void){
	PORTB = 0x00;
	PORTD = 0xff;
	DDRB = 0xff;		/* All output dir , 0=IN,1=OUT*/
	DDRD = 0xfe;		/* PD7(O) ~ PD2(O),PD1/TXD(O),PD0/RXD(I) = 0xFE */
}
void	init_timer(void){
	TCCR1B = 0x0d;		/* CTC1=1(Match clear), CK(4MHz)/1024 = 3.90625kHz : 256usec*/
	TCNT1 = 0;
	step_mot(t100msec);
}
void	init_uart(void){
	UBRRL = 25;      	/* 9600 bps for 4MHz */
	UCSRB = 0x98;     	/* RXCIE, RXEN, TXEN*/
}
void	send_char(char d){
	while(!(UCSRA & (1<<UDRE)));
	UDR = d;
}

BYTE	receive_char(void){
	while(!(UCSRA & (1 <<RXC)));
	return UDR;
}

void    send_string(char *p){
char    rd;
    while(rd = *p++) send_char(rd);
}
void    send_fstring(char flash *p){
char    rd;
    while(rd = *p++) send_char(rd);
}

void	step_mot(WORD val){
	if(val == 0){
		TIMSK &= ~(0x10);	/* OCIE1A = 0 ; Timer/Counter1 OCM Interrupt Disable */
		PORTB &= 0xf0;		/* Step_Mot_PIN All off(0) */
	}
	else{
		TCNT1 = 0;
		OCR1A = val;
		TIMSK |= 0x10;		/* OCIE1A = 1 ; Timer/Counter1 OCM Interrupt Enable */
	}	
}

interrupt [TIM1_COMPA] void TIMER1_COMPA_interrupt(void) 
{  
	t_cnt1++;
	PORTB = sm_data[t_cnt1 & 0x03];
}

interrupt [USART_RXC] void UART_RX_interrupt(void) 
{  
	rxd=UDR;
	switch(rxd){
		case '0': step_mot(0);
			putsf("\n\rStep Motor is =>STOP");
	  		break;
	  	case '1': step_mot(t500msec);
			putsf("\n\rStep Motor Increment time is =>500msec");
	  		break;
	  	case '2': step_mot(t200msec);
			putsf("\n\rStep Motor Increment time is =>200msec");
	  		break;
	  	case '3': step_mot(t100msec);
			putsf("\n\rStep Motor Increment time is =>100msec");
	  		break;
	  	case '4': step_mot(t50msec);
			putsf("\n\rStep Motor Increment time is =>50msec");
	  		break;
	  	case '5': step_mot(t20msec);
			putsf("\n\rStep Motor Increment time is =>20msec");
	  		break;
	  	case '6': step_mot(t10msec);
			putsf("\n\rStep Motor Increment time is =>10msec");
	  		break;
	  	case '7': step_mot(t5msec);
			putsf("\n\rStep Motor Increment time is =>5msec");
	  		break;
	  	case '8': step_mot(t2msec);
			putsf("\n\rStep Motor Increment time is =>2msec");
	  		break;
	  	case '9': step_mot(t1msec);
			putsf("\n\rStep Motor Increment time is =>1msec");
	  		break;
		default : step_mot(0);
			putsf("\n\rStep Motor is =>STOP");
	  		break;
	}
}
void main(void){
	init_port();
	init_uart();
	init_timer();
	putsf(message);

	#asm	("sei")		// global enable interrupts
	while(1){
		m_cnt++;
		delay_ms(1000);
	}
}
