/*********************************************
file name: led_intt.c
BY RealSYS KCO	2000.12.31

function: LED shift display using timer1 overflow interrupt
reference file : CodeVisionAVR LED Example file

AVR Chip: tiny2313 (8MHz)
cable connection:
	CN17(1~8):PB <==> CN28(1~8):LED
*********************************************/
// CodeVisionAVR C Compiler
// (C) 1998-2003 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the ATtiny2313


#pragma used+
#pragma used+
sfrb DIDR=1;
sfrb UBRRH=2;
sfrb UCSRC=3;
sfrb ACSR=8;
sfrb UBRRL=9;
sfrb UCSRB=0xa;
sfrb UCSRA=0xb;
sfrb UDR=0xc;
sfrb USICR=0xd;
sfrb USISR=0xe;
sfrb USIDR=0xf;
sfrb PIND=0x10;
sfrb DDRD=0x11;
sfrb PORTD=0x12;
sfrb GPIOR0=0x13;
sfrb GPIOR1=0x14;
sfrb GPIOR2=0x15;
sfrb PINB=0x16;
sfrb DDRB=0x17;
sfrb PORTB=0x18;
sfrb PINA=0x19;
sfrb DDRA=0x1a;
sfrb PORTA=0x1b;
sfrb EECR=0x1c;
sfrb EEDR=0x1d;
sfrb EEAR=0x1e;
sfrb PCMSK=0x20;
sfrb WDTCR=0x21;
sfrb TCCR1C=0x22;
sfrb GTCCR=0x23;
sfrb ICR1L=0x24;
sfrb ICR1H=0x25;
sfrw ICR1=0x24;   // 16 bit access
sfrb CLKPR=0x26;
sfrb OCR1BL=0x28;
sfrb OCR1BH=0x29;
sfrw OCR1B=0x28;   // 16 bit access
sfrb OCR1AL=0x2a;
sfrb OCR1AH=0x2b;
sfrw OCR1A=0x2a;   // 16 bit access
sfrb TCNT1L=0x2c;
sfrb TCNT1H=0x2d;
sfrw TCNT1=0x2c;  // 16 bit access
sfrb TCCR1B=0x2e;
sfrb TCCR1A=0x2f;
sfrb TCCR0A=0x30;
sfrb OSCCAL=0x31;
sfrb TCNT0=0x32;
sfrb TCCR0B=0x33;
sfrb MCUSR=0x34;
sfrb MCUCR=0x35;
sfrb OCR0A=0x36;
sfrb SPMCSR=0x37;
sfrb TIFR=0x38;
sfrb TIMSK=0x39;
sfrb EIFR=0x3a;
sfrb GIMSK=0x3b;
sfrb OCR0B=0x3c;
sfrb SPL=0x3d;
sfrb SREG=0x3f;
#pragma used-

// Interrupt vectors definitions


// for compatibility with the interrupt vector names from Atmel's datasheet


// Needed by the power management functions (sleep.h)
#asm
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x20
	.EQU __sm_mask=0x50
	.EQU __sm_powerdown=0x10
	.EQU __sm_standby=0x40
	.SET power_ctrl_reg=mcucr
	#endif
#endasm



// quartz crystal frquency [Hz]

// moving LED frequency [Hz]

// the LED on PORTB output 0 will be on
unsigned char led_status=0xfe;

// TIMER1 overflow interrupt service routine
interrupt [6] void timer1_overflow(void)
{
	// preset again TIMER1
	TCNT1=0x10000-(8000000/1024/10);
	// move the LED
	led_status<<=1;
	led_status|=1;
	if (led_status==0xff) led_status=0xfe;
	
	// turn on the LED
	PORTB=led_status;
}

void main(void)
{
	// set the I/O ports
	// all PORTB pins are outputs
	DDRB=0xff;
	// turn on the first LED
	PORTB=led_status;
	
	// init TIMER1
	// TIMER1 is disconnected from pin OC1
	// no PWM
	TCCR1A=0;

	// TIMER1 clock is xtal/1024
	TCCR1B=5;

	// preset TIMER1
	TCNT1=0x10000-(8000000/1024/10);

	// clear TIMER1 interrupts flags
	TIFR=0;

	// enable TIMER1 overflow interrupt
	TIMSK=0x80;

	// all other interrupt sources are disabled
	GIMSK=0;
	
	// global enable interrupts
	#asm
	    sei
	#endasm
	
	// the rest is done by TIMER1 overflow interrupts
	while (1);
}
