/*********************************************
file name: dc_mot.c
BY RealSYS KCO	2001.1.1

function: DC Motor Speed Control using PWM duty
	PC Comm. Program : '0'=STOP ~ '9'=High speed

AVR Chip: tiny2313 (8MHz)

cable connection:
	CN17(4):PB <==> CN26(1):DC_mot

	RS232(20P-CPU) <==> PC COM Port

	CN25 <==> DC Motor
*********************************************/
#include <tiny2313.h>
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x20
	.EQU __sm_mask=0x50
	.EQU __sm_powerdown=0x10
	.EQU __sm_standby=0x40
	.SET power_ctrl_reg=mcucr
	#endif
#include <delay.h>
#include <stdio.h>

#define	BYTE	unsigned char
#define	WORD	unsigned int

/* UART Status Register */
#define    RXC     7
#define    TXC     6
#define    UDRE    5
#define    FE      4
#define    OVR     3

BYTE	rxd;
int 	m_cnt=0;
int		t_cnt0=0;

flash BYTE fbuf[]={"\n\rPWM output value is =>"};
flash BYTE message[]={"\n\rPress KEY : '0'(Low) ~ '9'(High)"};

void  init_port(void){
	PORTB = 0xff;
	PORTD = 0xff;
	DDRB = 0xff;		/* All output dir , 0=IN,1=OUT*/
	DDRD = 0xfe;		/* PD6(O)~PD2(O),PD1/TXD(O),PD0/RXD(I) = 0xFE */
}
void	init_timer(void){
	TCCR0B = 4;			/* CK(8MHz)/256 = 31.250kHz : 32usec*/
	TIMSK |= 2;			/* TOIE0 = 1 */
	TCNT0 = -32;		/* 32usec * 32 = 1.024 msec period */

	TCCR1B = 0x01;		/* Timer1 Clock = CK */
	TCCR1A = 0x81;		/* 8bit NON-INV PWM */
	OCR1AH = 0;
	OCR1AL = 50;

}
void	init_uart(void){
    UCSRA = 0xe0;        /* U2X=0 */
	UBRRL = 51;      	/* 9600 bps for 8MHz */
	UCSRB = 0x98;     	/* RXCIE, RXEN, TXEN*/
}
void	send_char(char d){
	while(!(UCSRA & (1<<UDRE)));
	UDR = d;
}

BYTE	receive_char(void){
	while(!(UCSRA & (1 <<RXC)));
	return UDR;
}

void    send_string(char flash *p){
char    rd;
    while(rd = *p++) send_char(rd);
}

interrupt [TIM0_OVF] void TIMER0_OVF0_interrupt(void)
{
	TCNT0 = -32;		/* 32usec * 32 = 1.024 msec period */
	t_cnt0++;
}

interrupt [USART_RXC] void UART_RX_interrupt(void)
{
	rxd=UDR;
	send_string(fbuf);
	switch(rxd){
		case '0': OCR1AL = 0;
			putsf("0");
	  		break;
	  	case '1': OCR1AL = 25;
			putsf("25");
	  		break;
	  	case '2': OCR1AL = 50;
			putsf("50");
	  		break;
	  	case '3': OCR1AL = 75;
			putsf("75");
	  		break;
	  	case '4': OCR1AL = 100;
			putsf("100");
	  		break;
	  	case '5': OCR1AL = 125;
			putsf("125");
	  		break;
	  	case '6': OCR1AL = 150;
			putsf("150");
	  		break;
	  	case '7': OCR1AL = 175;
			putsf("175");
	  		break;
	  	case '8': OCR1AL = 200;
			putsf("200");
	  		break;
	  	case '9': OCR1AL = 255;
			putsf("255");
	  		break;
		default : OCR1AL = 0;
			putsf("0");
	  		break;
	}
}

void main(void){
	init_port();
	init_uart();
	init_timer();
	send_string(message);

#asm	("sei")		// global enable interrupts

	while(1){
		m_cnt++;
		delay_ms(1000);
		printf("\n\rMain counter(m_cnt) value is %d(0x%04X)",m_cnt,m_cnt);
	}
}
