/*********************************************
file name: seg.c
BY RealSYS KCO	2001.1.1

function: 7 segment display

AVR Chip: tiny2313 (8MHz)
cable connection:
	CN17(1~8):PB <==> CN29(1~8):SEG_Data
	CN18(4~7):PD <==> CN27(1~4):SEG_Scan
*********************************************/
// CodeVisionAVR C Compiler
// (C) 1998-2003 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the ATtiny2313


#pragma used+
#pragma used+
sfrb DIDR=1;
sfrb UBRRH=2;
sfrb UCSRC=3;
sfrb ACSR=8;
sfrb UBRRL=9;
sfrb UCSRB=0xa;
sfrb UCSRA=0xb;
sfrb UDR=0xc;
sfrb USICR=0xd;
sfrb USISR=0xe;
sfrb USIDR=0xf;
sfrb PIND=0x10;
sfrb DDRD=0x11;
sfrb PORTD=0x12;
sfrb GPIOR0=0x13;
sfrb GPIOR1=0x14;
sfrb GPIOR2=0x15;
sfrb PINB=0x16;
sfrb DDRB=0x17;
sfrb PORTB=0x18;
sfrb PINA=0x19;
sfrb DDRA=0x1a;
sfrb PORTA=0x1b;
sfrb EECR=0x1c;
sfrb EEDR=0x1d;
sfrb EEAR=0x1e;
sfrb PCMSK=0x20;
sfrb WDTCR=0x21;
sfrb TCCR1C=0x22;
sfrb GTCCR=0x23;
sfrb ICR1L=0x24;
sfrb ICR1H=0x25;
sfrw ICR1=0x24;   // 16 bit access
sfrb CLKPR=0x26;
sfrb OCR1BL=0x28;
sfrb OCR1BH=0x29;
sfrw OCR1B=0x28;   // 16 bit access
sfrb OCR1AL=0x2a;
sfrb OCR1AH=0x2b;
sfrw OCR1A=0x2a;   // 16 bit access
sfrb TCNT1L=0x2c;
sfrb TCNT1H=0x2d;
sfrw TCNT1=0x2c;  // 16 bit access
sfrb TCCR1B=0x2e;
sfrb TCCR1A=0x2f;
sfrb TCCR0A=0x30;
sfrb OSCCAL=0x31;
sfrb TCNT0=0x32;
sfrb TCCR0B=0x33;
sfrb MCUSR=0x34;
sfrb MCUCR=0x35;
sfrb OCR0A=0x36;
sfrb SPMCSR=0x37;
sfrb TIFR=0x38;
sfrb TIMSK=0x39;
sfrb EIFR=0x3a;
sfrb GIMSK=0x3b;
sfrb OCR0B=0x3c;
sfrb SPL=0x3d;
sfrb SREG=0x3f;
#pragma used-

// Interrupt vectors definitions


// for compatibility with the interrupt vector names from Atmel's datasheet


// Needed by the power management functions (sleep.h)
#asm
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x20
	.EQU __sm_mask=0x50
	.EQU __sm_powerdown=0x10
	.EQU __sm_standby=0x40
	.SET power_ctrl_reg=mcucr
	#endif
#endasm


// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.


#pragma used+
#pragma used+

void delay_us(unsigned int n);
void delay_ms(unsigned int n);

#pragma used-



int m_cnt=0;
int	t_cnt=0;

unsigned char 	seg_hex[] = /* HEX font */
	{0xc0,0xf9,0xa4,0xb0,0x99,0x92,0x82,0xf8,0x80,0x90,0x88,0x83,0xc6,0xa1,0x86,0x8e};
unsigned char 	seg_blank = 0xff;		/* ' ' */
unsigned char 	seg_bar = 0xbf;			/* '-' */
unsigned char 	seg_all_on = 0x00;		/* '8.' */
unsigned char 	seg_t = 0x87;			/* 't' */
unsigned char 	seg_scan[4] = { 0xf7,0xef,0xdf,0xbf };
unsigned char	seg_data[4]= { 0xbf,0xbf,0xbf,0xbf };
unsigned char	scan_cnt=0;	

void  init_port(void){          
	PORTD = 0xff;
	DDRB = 0xff;	/* All output dir , 0=IN,1=OUT*/
	DDRD = 0xfe;	/* PD6(O),PD5(O),PD4(O), PD3(O),PD2(O),PD1/TXD(O),PD0/RXD(I) = 0xFE */
}
void	init_timer(void){
	TCCR0B = 4;		/* CK(8MHz)/256 = 31.250kHz : 32usec*/
	TIMSK = 2;		/* TOIE0 = 1 */
	TCNT0 = -32;	/* 32usec * 32 = 1.024 msec period */
}
interrupt [7] void TIMER0_OVF0_interrupt(void){ 
	TCNT0 = -32;	/* 32usec * 32 = 1.024 msec period */
	t_cnt++;

	/* 7segment */
	(PORTD = 0xff);					/* SEG OFF */
	(PORTB = seg_data[scan_cnt]);	/* Seg Data out */
	(PORTD = seg_scan[scan_cnt]); 	/* Seg Scan out */
	
	scan_cnt++;					
	if(scan_cnt > 3) scan_cnt = 0;
}
void	disp_seg4(unsigned int d){
	seg_data[0] = seg_hex[(d>>12) & 0x0f];
	seg_data[1] = seg_hex[(d>>8) & 0x0f];
	seg_data[2] = seg_hex[(d>>4) & 0x0f];
	seg_data[3] = seg_hex[d & 0x0f];
}

void main(void){    
//long	i;
	init_port();
	init_timer();

// global enable interrupts
#asm
    sei
#endasm
	while(1){
		m_cnt++;
		disp_seg4(m_cnt);    
		delay_ms(500);
	}
}
