/*********************************************
file name: step_mot.c
BY RealSYS KCO	2001.1.1

function: STEP Motor Control using timer compare match
	PC Comm. Program : '0'=STOP ~ '9'=High speed
	
AVR Chip: tiny2313 (8MHz)

cable connection:
	CN17(1~4):PB <==> CN23(1~8):STEP_mot

	RS232(20P-CPU) <==> PC COM Port
	
	CN19 <==> Step Motor
*********************************************/
// CodeVisionAVR C Compiler
// (C) 1998-2003 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the ATtiny2313


#pragma used+
#pragma used+
sfrb DIDR=1;
sfrb UBRRH=2;
sfrb UCSRC=3;
sfrb ACSR=8;
sfrb UBRRL=9;
sfrb UCSRB=0xa;
sfrb UCSRA=0xb;
sfrb UDR=0xc;
sfrb USICR=0xd;
sfrb USISR=0xe;
sfrb USIDR=0xf;
sfrb PIND=0x10;
sfrb DDRD=0x11;
sfrb PORTD=0x12;
sfrb GPIOR0=0x13;
sfrb GPIOR1=0x14;
sfrb GPIOR2=0x15;
sfrb PINB=0x16;
sfrb DDRB=0x17;
sfrb PORTB=0x18;
sfrb PINA=0x19;
sfrb DDRA=0x1a;
sfrb PORTA=0x1b;
sfrb EECR=0x1c;
sfrb EEDR=0x1d;
sfrb EEAR=0x1e;
sfrb PCMSK=0x20;
sfrb WDTCR=0x21;
sfrb TCCR1C=0x22;
sfrb GTCCR=0x23;
sfrb ICR1L=0x24;
sfrb ICR1H=0x25;
sfrw ICR1=0x24;   // 16 bit access
sfrb CLKPR=0x26;
sfrb OCR1BL=0x28;
sfrb OCR1BH=0x29;
sfrw OCR1B=0x28;   // 16 bit access
sfrb OCR1AL=0x2a;
sfrb OCR1AH=0x2b;
sfrw OCR1A=0x2a;   // 16 bit access
sfrb TCNT1L=0x2c;
sfrb TCNT1H=0x2d;
sfrw TCNT1=0x2c;  // 16 bit access
sfrb TCCR1B=0x2e;
sfrb TCCR1A=0x2f;
sfrb TCCR0A=0x30;
sfrb OSCCAL=0x31;
sfrb TCNT0=0x32;
sfrb TCCR0B=0x33;
sfrb MCUSR=0x34;
sfrb MCUCR=0x35;
sfrb OCR0A=0x36;
sfrb SPMCSR=0x37;
sfrb TIFR=0x38;
sfrb TIMSK=0x39;
sfrb EIFR=0x3a;
sfrb GIMSK=0x3b;
sfrb OCR0B=0x3c;
sfrb SPL=0x3d;
sfrb SREG=0x3f;
#pragma used-

// Interrupt vectors definitions


// for compatibility with the interrupt vector names from Atmel's datasheet


// Needed by the power management functions (sleep.h)
#asm
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x20
	.EQU __sm_mask=0x50
	.EQU __sm_powerdown=0x10
	.EQU __sm_standby=0x40
	.SET power_ctrl_reg=mcucr
	#endif
#endasm


// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.


#pragma used+
#pragma used+

void delay_us(unsigned int n);
void delay_ms(unsigned int n);

#pragma used-

// CodeVisionAVR C Compiler
// (C) 1998-2006 Pavel Haiduc, HP InfoTech S.R.L.

// Prototypes for standard I/O functions


// CodeVisionAVR C Compiler
// (C) 1998-2002 Pavel Haiduc, HP InfoTech S.R.L.

// Variable length argument list macros



typedef char *va_list;
typedef char *va_list;
typedef char *va_list;





#pragma used+

char getchar(void);
void putchar(char c);
void puts(char *str);
void putsf(char flash *str);

char *gets(char *str,unsigned char len);

void printf(char flash *fmtstr,...);
void sprintf(char *str, char flash *fmtstr,...);
void snprintf(char *str, unsigned char size, char flash *fmtstr,...);
void vprintf (char flash * fmtstr, va_list argptr);
void vsprintf (char *str, char flash * fmtstr, va_list argptr);
void vsnprintf (char *str, unsigned char size, char flash * fmtstr, va_list argptr);
signed char scanf(char flash *fmtstr,...);
signed char sscanf(char *str, char flash *fmtstr,...);




                                   

            


#pragma used-

#pragma library stdio.lib




/* UART Status Register */



unsigned char	rxd;   
int 	m_cnt=0;
int		t_cnt0=0,t_cnt1=0;

flash unsigned char fbuf[]={"\n\rStep Motor Increment time is =>"};
flash unsigned char message[]={"\n\rPress KEY : '0'(STOP) ~ '9'(High)"};

/************ Step motor ***************/
/*BYTE	sm_data[] = { 0x0c,0x09,0x03,0x06 };*/
unsigned char	sm_data[] = { 0x01,0x02,0x04,0x08 };

void	step_mot(unsigned int val);
void  init_port(void){
	PORTB = 0x00;
	PORTD = 0xff;
	DDRB = 0xff;		/* All output dir , 0=IN,1=OUT*/
	DDRD = 0xfe;		/* PD6(O) ~ PD2(O),PD1/TXD(O),PD0/RXD(I) = 0xFE */
}
void	init_timer(void){
	TCCR0B = 4;			/* CK(8MHz)/256 = 31.250kHz : 32usec*/
	TIMSK |= 2;			/* TOIE0 = 1 */
	TCNT0 = -32;		/* 32usec * 32 = 1.024 msec period */

	TCCR1B = 0x0d;		/* CTC1=1(Match clear), CK(8MHz)/1024 = 7.8125kHz : 128usec*/
	TCNT1H = 0;	TCNT1L = 0;
	step_mot(781		);
}
void	init_uart(void){
	UCSRA.1=0;
	UBRRL = 51;      	/* 9600 bps for 8MHz */
	UCSRB = 0x98;     	/* RXCIE, RXEN, TXEN*/
}
void	send_char(char d){
	while(!(USR & (1<<5)));
	UDR = d;
}

unsigned char	receive_char(void){
	while(!(USR & (1 <<7)));
	return UDR;
}

void    send_string(char *p){
char    rd;
    while(rd = *p++) send_char(rd);
}
void    send_fstring(char flash *p){
char    rd;
    while(rd = *p++) send_char(rd);
}

void	step_mot(unsigned int val){
	if(val == 0){
		TIMSK &= ~(0x40);	/* OCIE1A = 0 ; Timer/Counter1 OCM Interrupt Disable */
		PORTB &= 0xf0;		/* Step_Mot_PIN All off(0) */
	}
	else{
//		TCNT1H = 0;	TCNT1L = 0;       
		TCNT1 = 0;
		
//		OCR1AH = HI_BYTE(val);		OCR1AL = LO_BYTE(val);
		OCR1 = val;
		TIMSK |= 0x40;		/* OCIE1A = 1 ; Timer/Counter1 OCM Interrupt Enable */
	}	
}

interrupt [7] void TIMER0_OVF0_interrupt(void) 
{  
	TCNT0 = -32;		/* 32usec * 32 = 1.024 msec period */
	t_cnt0++;
}
interrupt [TIM1_COMP] void TIMER1_COMP1_interrupt(void) 
{  
	t_cnt1++;
	PORTB = sm_data[t_cnt1 & 0x03];
}

interrupt [8] void UART_RX_interrupt(void) 
{  
	rxd=UDR;
	putsf(fbuf);
	switch(rxd){
		case '0': step_mot(0);
			send_fstring("STOP");
	  		break;
	  	case '1': step_mot(3906	);
			send_fstring("500msec");
	  		break;
	  	case '2': step_mot(1563	);
			send_fstring("200msec");
	  		break;
	  	case '3': step_mot(781		);
			send_fstring("100msec");
	  		break;
	  	case '4': step_mot(391		);
			send_fstring("50msec");
	  		break;
	  	case '5': step_mot(156		);
			send_fstring("20msec");
	  		break;
	  	case '6': step_mot(78		);
			send_fstring("10msec");
	  		break;
	  	case '7': step_mot(39		);
			send_fstring("5msec");
	  		break;
	  	case '8': step_mot(16		);
			send_fstring("2msec");
	  		break;
	  	case '9': step_mot(8		);
			send_fstring("1msec");
	  		break;
		default : step_mot(0);
			send_fstring("STOP");
	  		break;
	}
}
void main(void){
	init_port();
	init_uart();
	init_timer();
	putsf(message);

	#asm	("sei")		// global enable interrupts
	while(1){
		m_cnt++;
		delay_ms(1000);
	}
}
